
/*!
 ***********************************************************************
 * \file image.c
 *
 * \brief
 *    Decode a Slice
 *
 * \author
 *    Main contributors (see contributors.h for copyright, address and affiliation details)
 *    - Inge Lille-Langoy               <inge.lille-langoy@telenor.com>
 *    - Rickard Sjoberg                 <rickard.sjoberg@era.ericsson.se>
 *    - Jani Lainema                    <jani.lainema@nokia.com>
 *    - Sebastian Purreiter             <sebastian.purreiter@mch.siemens.de>
 *    - Byeong-Moon Jeon                <jeonbm@lge.com>
 *    - Thomas Wedi                     <wedi@tnt.uni-hannover.de>
 *    - Gabi Blaettermann               <blaetter@hhi.de>
 *    - Ye-Kui Wang                     <wyk@ieee.org>
 *    - Antti Hallapuro                 <antti.hallapuro@nokia.com>
 *    - Alexis Tourapis                 <alexismt@ieee.org>
 ***********************************************************************
 */

#include "contributors.h"

#include <math.h>
#include <stdlib.h>
#include <time.h>
#include <sys/timeb.h>
#include <string.h>
#include <assert.h>

#include "global.h"
#include "errorconcealment.h"
#include "image.h"
#include "mbuffer.h"
#include "fmo.h"
#include "parsetcommon.h"
#include "parset.h"
#include "header.h"
#include "rtp.h"
#include "sei.h"
#include "output.h"
#include "mb_access.h"
#include "annexb.h"

#include "vlc.h"

pic_parameter_set_rbsp_t *active_pps;
seq_parameter_set_rbsp_t *active_sps;

extern int  FileFormat;
extern int g_new_frame;

extern StorablePicture **listX[2];

StorablePicture *dec_picture;
int  frame_no;
// global picture format dependend buffers, mem allocation in decod.c ******************
/************************************************************************/
// by niepeng 20071024
// ڴ滺ȡSlice
/************************************************************************/
int decode_one_frame(struct img_par *img,char* buf,int size,int &current_position)
{
  int current_header;
  Slice *currSlice = img->currentSlice;
  int ercStartMB;
  int ercSegment;

  img->current_slice_nr = 0;
  img->current_mb_nr = -4711;     // initialized to an impossible value for debugging -- correct value is taken from slice header
  currSlice->next_header = -8888; // initialized to an impossible value for debugging -- correct value is taken from slice header
  img->num_dec_mb = 0;

  //int current_position=0;		//ǰλ
  while ((currSlice->next_header != EOS && currSlice->next_header != SOP))
  {
   // current_header = read_new_slice();//sliceļ޸
	current_header=read_new_slice(buf,size,current_position);
    if (current_header == EOS)
      return EOS;

    //decode_frame_slice(img, current_header);
	Slice *currSlice = img->currentSlice;
	// init new frame
	if (current_header == SOP)
		  init_frame(img);
	// decode main slice information
	if ((current_header == SOP || current_header == SOS) && currSlice->ei_flag == 0)
	{
		//decode_one_slice
		Boolean end_of_slice = FALSE;
		img->cod_counter=-1;
		while (end_of_slice == FALSE) // loop over macroblocks
		{
			// Initializes the current macroblock
			start_macroblock(img, img->current_mb_nr);
			// Get the syntax elements from the NAL
			read_one_macroblock(img);
			decode_one_macroblock(img);
			end_of_slice=exit_macroblock(img,1);
		}
	}

    img->current_slice_nr++;
  }//end while

  //deblocking for frame
  DeblockFrame( img, dec_picture->imgY, dec_picture->imgUV ) ;

  store_picture_in_dpb(dec_picture);
  dec_picture=NULL;

  g_new_frame=1;

  //! this is always true at the beginning of a frame
  ercStartMB = 0;
  ercSegment = 0;

  post_poc( img );                    // POC200301


  //! TO 19.11.2001 Known Problem: for init_frame we have to know the picture type of the actual frame
  //! in case the first slice of the P-Frame following the I-Frame was lost we decode this P-Frame but 
  //! do not write it because it was assumed to be an I-Frame in init_frame. So we force the decoder to
  //! guess the right picture type. This is a hack a should be removed by the time there is a clean
  //! solution where we do not have to know the picture type for the function init_frame.
  if(img->type == I_SLICE)
	  img->type = P_SLICE;
  //! End TO 19.11.2001

  img->number++;

  //exit_frame(img);
  img->pre_frame_num = img->frame_num;


  img->current_mb_nr = -4712;   // impossible value for debugging, StW
  img->current_slice_nr = 0;

  return (SOP);
}
/*!
 ***********************************************************************
 * \brief
 *    decodes one I- or P-frame
 *
 ***********************************************************************
 */

int decode_one_frame(struct img_par *img)
{
  int current_header;
  Slice *currSlice = img->currentSlice;
  int ercStartMB;
  int ercSegment;

  img->current_slice_nr = 0;
  img->current_mb_nr = -4711;     // initialized to an impossible value for debugging -- correct value is taken from slice header
  currSlice->next_header = -8888; // initialized to an impossible value for debugging -- correct value is taken from slice header
  img->num_dec_mb = 0;

  while ((currSlice->next_header != EOS && currSlice->next_header != SOP))
  {
    current_header = read_new_slice();

    if (current_header == EOS)
      return EOS;

    //decode_frame_slice(img, current_header);
	Slice *currSlice = img->currentSlice;
	// init new frame
	if (current_header == SOP)
		  init_frame(img);
	// decode main slice information
	if ((current_header == SOP || current_header == SOS) && currSlice->ei_flag == 0)
	{
		//decode_one_slice
		Boolean end_of_slice = FALSE;
		img->cod_counter=-1;
		while (end_of_slice == FALSE) // loop over macroblocks
		{
			// Initializes the current macroblock
			start_macroblock(img, img->current_mb_nr);
			// Get the syntax elements from the NAL
			read_one_macroblock(img);
			decode_one_macroblock(img);
			end_of_slice=exit_macroblock(img,1);
		}
	}

    img->current_slice_nr++;
  }//end while

  //deblocking for frame
  DeblockFrame( img, dec_picture->imgY, dec_picture->imgUV ) ;

  store_picture_in_dpb(dec_picture);
  dec_picture=NULL;

  g_new_frame=1;

  //! this is always true at the beginning of a frame
  ercStartMB = 0;
  ercSegment = 0;

  post_poc( img );                    // POC200301


  //! TO 19.11.2001 Known Problem: for init_frame we have to know the picture type of the actual frame
  //! in case the first slice of the P-Frame following the I-Frame was lost we decode this P-Frame but 
  //! do not write it because it was assumed to be an I-Frame in init_frame. So we force the decoder to
  //! guess the right picture type. This is a hack a should be removed by the time there is a clean
  //! solution where we do not have to know the picture type for the function init_frame.
  if(img->type == I_SLICE)
	  img->type = P_SLICE;
  //! End TO 19.11.2001

  img->number++;

  //exit_frame(img);
  img->pre_frame_num = img->frame_num;


  img->current_mb_nr = -4712;   // impossible value for debugging, StW
  img->current_slice_nr = 0;

  return (SOP);
}


/*!
 ************************************************************************
 * \brief
 *    Interpolation of 1/4 subpixel
 ************************************************************************
 */
void get_block(int ref_frame, StorablePicture **list, int x_pos, int y_pos, struct img_par *img, int block[4][4])
{

  int dx, dy;
  int i, j;
  int maxold_x,maxold_y;
  int result,pres_idx,mode_idx;
  int pres_x;
  int pres_y; 
  int tmp_res[4][9];
  static const int COEF[6] = {1, -5, 20, 20, -5, 1};

  dx = x_pos&3;
  dy = y_pos&3;
  mode_idx=block_mode[dy][dx];
  x_pos = (x_pos-dx)>>2;
  y_pos = (y_pos-dy)>>2;

  maxold_x = img->width-1;
  maxold_y = img->height-1;

  switch(mode_idx)
  {
  case 1:
	  for (j = 0; j < 4; j++){
		  pres_idx=Clip3(0,maxold_y,y_pos+j);
		  for (i = 0; i < 4; i++)
			  block[i][j] = list[ref_frame]->imgY[pres_idx][Clip3(0,maxold_x,x_pos+i)];
	  }
	  return;
  case 2:
 	  for (j = 0; j < 4; j++) {
		  pres_idx=Clip3(0,maxold_y,y_pos+j);
		  for (i = 0; i < 4; i++) {
			  result = 0;
			  result += list[ref_frame]->imgY[pres_idx][Clip3(0,maxold_x,x_pos+i-2)]*COEF[0]
					+list[ref_frame]->imgY[pres_idx][Clip3(0,maxold_x,x_pos+i-1)]*COEF[1]
					+list[ref_frame]->imgY[pres_idx][Clip3(0,maxold_x,x_pos+i)]*COEF[2]
					+list[ref_frame]->imgY[pres_idx][Clip3(0,maxold_x,x_pos+i+1)]*COEF[3]
					+list[ref_frame]->imgY[pres_idx][Clip3(0,maxold_x,x_pos+i+2)]*COEF[4]
					+list[ref_frame]->imgY[pres_idx][Clip3(0,maxold_x,x_pos+i+3)]*COEF[5];
			  block[i][j] = Clip1((result+16)>>5);
		  }
      }
	  return;
  case 3:
	  for (j = 0; j < 4; j++) {
		  pres_idx=Clip3(0,maxold_y,y_pos+j);
		  for (i = 0; i < 4; i++) {
				result = 0;
				result += list[ref_frame]->imgY[pres_idx][Clip3(0,maxold_x,x_pos+i-2)]*COEF[0]
					+list[ref_frame]->imgY[pres_idx][Clip3(0,maxold_x,x_pos+i-1)]*COEF[1]
					+list[ref_frame]->imgY[pres_idx][Clip3(0,maxold_x,x_pos+i)]*COEF[2]
					+list[ref_frame]->imgY[pres_idx][Clip3(0,maxold_x,x_pos+i+1)]*COEF[3]
					+list[ref_frame]->imgY[pres_idx][Clip3(0,maxold_x,x_pos+i+2)]*COEF[4]
					+list[ref_frame]->imgY[pres_idx][Clip3(0,maxold_x,x_pos+i+3)]*COEF[5];
				block[i][j] = (Clip1((result+16)>>5) + list[ref_frame]->imgY[pres_idx][Clip3(0,maxold_x,x_pos+i+dx/2)] +1 )/2;
		  }
      }
	  return;
  case 4:
	  for (i = 0; i < 4; i++) {
         for (j = 0; j < 4; j++){
			 pres_idx = Clip3(0,maxold_x,x_pos+i);
			result = 0;
            result += list[ref_frame]->imgY[Clip3(0,maxold_y,y_pos+j-2)][pres_idx]*COEF[0]
				+list[ref_frame]->imgY[Clip3(0,maxold_y,y_pos+j-1)][pres_idx]*COEF[1]
				+list[ref_frame]->imgY[Clip3(0,maxold_y,y_pos+j)][pres_idx]*COEF[2]
				+list[ref_frame]->imgY[Clip3(0,maxold_y,y_pos+j+1)][pres_idx]*COEF[3]
				+list[ref_frame]->imgY[Clip3(0,maxold_y,y_pos+j+2)][pres_idx]*COEF[4]
				+list[ref_frame]->imgY[Clip3(0,maxold_y,y_pos+j+3)][pres_idx]*COEF[5];
          block[i][j] = Clip1((result+16)>>5);
        }
      }
	  return;
  case 5:
	  for (i = 0; i < 4; i++) {
		  pres_idx = Clip3(0,maxold_x,x_pos+i);
		  for (j = 0; j < 4; j++){
			 result = 0;
			 result += list[ref_frame]->imgY[Clip3(0,maxold_y,y_pos+j-2)][pres_idx]*COEF[0]
				+list[ref_frame]->imgY[Clip3(0,maxold_y,y_pos+j-1)][pres_idx]*COEF[1]
				+list[ref_frame]->imgY[Clip3(0,maxold_y,y_pos+j)][pres_idx]*COEF[2]
				+list[ref_frame]->imgY[Clip3(0,maxold_y,y_pos+j+1)][pres_idx]*COEF[3]
				+list[ref_frame]->imgY[Clip3(0,maxold_y,y_pos+j+2)][pres_idx]*COEF[4]
				+list[ref_frame]->imgY[Clip3(0,maxold_y,y_pos+j+3)][pres_idx]*COEF[5];
			 block[i][j] = (Clip1((result+16)>>5) + list[ref_frame]->imgY[Clip3(0,maxold_y,y_pos+j+dy/2)][pres_idx] +1 )/2;
		  }
      }
	  return;
  case 6:
	  for (j = -2; j < 4+3; j++) {
		  pres_idx = Clip3(0,maxold_y,y_pos+j);
		  for (i = 0; i < 4; i++){
			  tmp_res[i][j+2] = 0;
			  tmp_res[i][j+2] += list[ref_frame]->imgY[pres_idx][Clip3(0,maxold_x,x_pos+i-2)]*COEF[0]
				  +list[ref_frame]->imgY[pres_idx][Clip3(0,maxold_x,x_pos+i-1)]*COEF[1]
				  +list[ref_frame]->imgY[pres_idx][Clip3(0,maxold_x,x_pos+i)]*COEF[2]
				  +list[ref_frame]->imgY[pres_idx][Clip3(0,maxold_x,x_pos+i+1)]*COEF[3]
				  +list[ref_frame]->imgY[pres_idx][Clip3(0,maxold_x,x_pos+i+2)]*COEF[4]
				  +list[ref_frame]->imgY[pres_idx][Clip3(0,maxold_x,x_pos+i+3)]*COEF[5];
		  }
      }

      for (j = 0; j < 4; j++) {
        for (i = 0; i < 4; i++) {
			result = 0,
            result += tmp_res[i][j]*COEF[0]
				+tmp_res[i][j+1]*COEF[1]
				+tmp_res[i][j+2]*COEF[2]
				+tmp_res[i][j+3]*COEF[3]
				+tmp_res[i][j+4]*COEF[4]
				+tmp_res[i][j+5]*COEF[5];
          block[i][j] = Clip1((result+512)>>10);
        } 
      }
	  return;
  case 7:
	  for (j = -2; j < 4+3; j++) {
		  pres_idx = Clip3(0,maxold_y,y_pos+j);
		  for (i = 0; i < 4; i++){
			  tmp_res[i][j+2] = 0;
			  tmp_res[i][j+2] += list[ref_frame]->imgY[pres_idx][Clip3(0,maxold_x,x_pos+i-2)]*COEF[0]
				  +list[ref_frame]->imgY[pres_idx][Clip3(0,maxold_x,x_pos+i-1)]*COEF[1]
				  +list[ref_frame]->imgY[pres_idx][Clip3(0,maxold_x,x_pos+i)]*COEF[2]
				  +list[ref_frame]->imgY[pres_idx][Clip3(0,maxold_x,x_pos+i+1)]*COEF[3]
				  +list[ref_frame]->imgY[pres_idx][Clip3(0,maxold_x,x_pos+i+2)]*COEF[4]
				  +list[ref_frame]->imgY[pres_idx][Clip3(0,maxold_x,x_pos+i+3)]*COEF[5];
		  }
      }

      for (j = 0; j < 4; j++) {
        for (i = 0; i < 4; i++) {
			result = 0,
            result += tmp_res[i][j]*COEF[0]
				+tmp_res[i][j+1]*COEF[1]
				+tmp_res[i][j+2]*COEF[2]
				+tmp_res[i][j+3]*COEF[3]
				+tmp_res[i][j+4]*COEF[4]
				+tmp_res[i][j+5]*COEF[5];
          block[i][j] = (Clip1((result+512)>>10) + Clip1((tmp_res[i][j+2+dy/2]+16)>>5) +1 )/2;
        } 
      }
 	  return;
  case 8:
	  for (i = -2; i < 4+3; i++) {
		  pres_idx = Clip3(0,maxold_x,x_pos+i);
		  for (j = 0; j < 4; j++){
			  tmp_res[j][i+2] = 0;
			  tmp_res[j][i+2] += list[ref_frame]->imgY[Clip3(0,maxold_y,y_pos+j-2)][pres_idx]*COEF[0]
					+ list[ref_frame]->imgY[Clip3(0,maxold_y,y_pos+j-1)][pres_idx]*COEF[1]
					+ list[ref_frame]->imgY[Clip3(0,maxold_y,y_pos+j)][pres_idx]*COEF[2]
					+ list[ref_frame]->imgY[Clip3(0,maxold_y,y_pos+j+1)][pres_idx]*COEF[3]
					+ list[ref_frame]->imgY[Clip3(0,maxold_y,y_pos+j+2)][pres_idx]*COEF[4]
					+ list[ref_frame]->imgY[Clip3(0,maxold_y,y_pos+j+3)][pres_idx]*COEF[5];
		  }
      }

      for (j = 0; j < 4; j++) {
        for (i = 0; i < 4; i++) {
			result = 0;
            result += tmp_res[j][i]*COEF[0]
				+tmp_res[j][i+1]*COEF[1]
				+tmp_res[j][i+2]*COEF[2]
				+tmp_res[j][i+3]*COEF[3]
				+tmp_res[j][i+4]*COEF[4]
				+tmp_res[j][i+5]*COEF[5];
			block[i][j] = (Clip1((result+512)>>10) + Clip1((tmp_res[j][2+i+dx/2]+16)>>5)+1)/2;
        }
      }
	  return;
  default:
	  for (j = 0; j < 4; j++) {
        for (i = 0; i < 4; i++) {
          pres_y = dy == 1 ? y_pos+j : y_pos+j+1;
          pres_y = Clip3(0,maxold_y,pres_y);
		  result = 0;
		  result += list[ref_frame]->imgY[pres_y][Clip3(0,maxold_x,x_pos+i-2)]*COEF[0]
			  +list[ref_frame]->imgY[pres_y][Clip3(0,maxold_x,x_pos+i-1)]*COEF[1]
			  +list[ref_frame]->imgY[pres_y][Clip3(0,maxold_x,x_pos+i)]*COEF[2]
			  +list[ref_frame]->imgY[pres_y][Clip3(0,maxold_x,x_pos+i+1)]*COEF[3]
			  +list[ref_frame]->imgY[pres_y][Clip3(0,maxold_x,x_pos+i+2)]*COEF[4]
			  +list[ref_frame]->imgY[pres_y][Clip3(0,maxold_x,x_pos+i+3)]*COEF[5];
          block[i][j] = Clip1((result+16)>>5);

          pres_x = dx == 1 ? x_pos+i : x_pos+i+1;
          pres_x = Clip3(0,maxold_x,pres_x);
		  result = 0;
          result += list[ref_frame]->imgY[Clip3(0,maxold_y,y_pos+j-2)][pres_x]*COEF[0]
			  +list[ref_frame]->imgY[Clip3(0,maxold_y,y_pos+j-1)][pres_x]*COEF[1]
			  +list[ref_frame]->imgY[Clip3(0,maxold_y,y_pos+j)][pres_x]*COEF[2]
			  +list[ref_frame]->imgY[Clip3(0,maxold_y,y_pos+j+1)][pres_x]*COEF[3]
			  +list[ref_frame]->imgY[Clip3(0,maxold_y,y_pos+j+2)][pres_x]*COEF[4]
			  +list[ref_frame]->imgY[Clip3(0,maxold_y,y_pos+j+3)][pres_x]*COEF[5];
          block[i][j] = (block[i][j] + Clip1((result+16)>>5) +1 ) / 2;
        }
      }
	return;
  }
}


static void reorder_lists(int currSliceType, Slice * currSlice)
{

  if (currSliceType != I_SLICE)
  {
    if (currSlice->ref_pic_list_reordering_flag_l0)
    {
      reorder_ref_pic_list(listX[0], &listXsize[0], 
                           img->num_ref_idx_l0_active - 1, 
                           currSlice->remapping_of_pic_nums_idc_l0, 
                           currSlice->abs_diff_pic_num_minus1_l0, 
                           currSlice->long_term_pic_idx_l0);
    }
    if (NULL == listX[0][img->num_ref_idx_l0_active-1])
    {
      error("number of entries in list 0 smaller than num_ref_idx_l0_active_minus1",500);
    }
    // that's a definition
    listXsize[0] = img->num_ref_idx_l0_active;
  }

  free_ref_pic_list_reordering_buffer(currSlice);
}

//add by niepeng 20071025
//ӻslice
int read_new_slice(char* buf,int size,int &current_position)
{
  NALU_t *nalu = AllocNALU(MAX_CODED_FRAME_SIZE);
  int current_header;
  int ret;
  int BitsUsedByHeader;
  Slice *currSlice = img->currentSlice;
  Bitstream *currStream;
  int newframe;

  int slice_id_a, slice_id_b, slice_id_c;
  int redundant_pic_cnt_b, redundant_pic_cnt_c;
  long ftell_position;

  //int current_size=size;
  while (1)
  {
    ftell_position = current_position;//ftell(bits);
//	buf=buf+current_position;
	
    if (FileFormat == PAR_OF_ANNEXB)
     // ret=GetAnnexbNALU (nalu);
	 ret=GetAnnexbNALU(nalu,buf,size,current_position);
    else
      ret=GetRTPNALU (nalu);

	//NALUtoRBSP(nalu);
   assert (nalu != NULL);
   //EBSPtoRBSP (nalu->buf, nalu->len) ; 
    
    if (ret < 0)
		exit (-1);
    else if (ret == 0)
    {
        return EOS;
    }

    switch (nalu->nal_unit_type)
    {
      case NALU_TYPE_SLICE:
      case NALU_TYPE_IDR:
        img->idr_flag = (nalu->nal_unit_type == NALU_TYPE_IDR);
        img->nal_reference_idc = nalu->nal_reference_idc;
        img->disposable_flag = (nalu->nal_reference_idc == NALU_PRIORITY_DISPOSABLE);
        currSlice->dp_mode = PAR_DP_1;
        currSlice->max_part_nr = 1;
        currSlice->ei_flag = 0;
        currStream = currSlice->partArr[0].bitstream;
        currStream->ei_flag = 0;
        currStream->frame_bitoffset = 0;
        memcpy (currStream->streamBuffer, &nalu->buf[1], nalu->len-1);
        currStream->bitstream_length = RBSPtoSODB(currStream->streamBuffer, nalu->len-1);

        // Some syntax of the Slice Header depends on the parameter set, which depends on
        // the parameter set ID of the SLice header.  Hence, read the pic_parameter_set_id
        // of the slice header first, then setup the active parameter sets, and then read
        // the rest of the slice header
        BitsUsedByHeader = FirstPartOfSliceHeader();
        UseParameterSet (currSlice->pic_parameter_set_id);
        BitsUsedByHeader+= RestOfSliceHeader ();

        FmoInit (active_pps, active_sps);

        init_lists(img->type);
        reorder_lists (img->type, img->currentSlice);

        // From here on, active_sps, active_pps and the slice header are valid
        img->current_mb_nr = currSlice->start_mb_nr;

        if (img->tr_old != img->ThisPOC)
        {
          newframe=1;
          img->tr_old = img->ThisPOC;
		  current_header = SOP;
        }
        else
		{
          newframe = 0;
		  current_header = SOS;
		}
		g_new_frame=0;

       FreeNALU(nalu);
        return current_header;
        break;
      case NALU_TYPE_DPA:
        //! The state machine here should follow the same ideas as the old readSliceRTP()
        //! basically:
        //! work on DPA (as above)
        //! read and process all following SEI/SPS/PPS/PD/Filler NALUs
        //! if next video NALU is dpB, 
        //!   then read and check whether it belongs to DPA, if yes, use it
        //! else
        //!   ;   // nothing
        //! read and process all following SEI/SPS/PPS/PD/Filler NALUs
        //! if next video NALU is dpC
        //!   then read and check whether it belongs to DPA (and DPB, if present), if yes, use it, done
        //! else
        //!   use the DPA (and the DPB if present)

        /* 
            LC: inserting the code related to DP processing, mainly copying some of the parts
            related to NALU_TYPE_SLICE, NALU_TYPE_IDR.
        */


        img->idr_flag          = (nalu->nal_unit_type == NALU_TYPE_IDR);
        img->nal_reference_idc = nalu->nal_reference_idc;
        img->disposable_flag   = (nalu->nal_reference_idc == NALU_PRIORITY_DISPOSABLE);
        currSlice->dp_mode     = PAR_DP_3;
        currSlice->max_part_nr = 3;
        currSlice->ei_flag     = 0;
        currStream             = currSlice->partArr[0].bitstream;
        currStream->ei_flag    = 0;
        currStream->frame_bitoffset = 0;
        memcpy (currStream->streamBuffer, &nalu->buf[1], nalu->len-1);
        currStream->bitstream_length = RBSPtoSODB(currStream->streamBuffer, nalu->len-1);
        
        BitsUsedByHeader     = FirstPartOfSliceHeader();
        UseParameterSet (currSlice->pic_parameter_set_id);
        BitsUsedByHeader    += RestOfSliceHeader ();
        
        FmoInit (active_pps, active_sps);
        
        init_lists(img->type);
        reorder_lists (img->type, img->currentSlice);

        // From here on, active_sps, active_pps and the slice header are valid
        img->current_mb_nr = currSlice->start_mb_nr;

        if (img->tr_old != img->ThisPOC)
        {
          newframe=1;
          img->tr_old = img->ThisPOC;
		  current_header = SOP;
        }
        else
		{
          newframe = 0;
		  current_header = SOS;
		}
		g_new_frame=0;

        /* 
           LC:
              Now I need to read the slice ID, which depends on the value of 
              redundant_pic_cnt_present_flag (pag.49). 
        */

        slice_id_a  = ue_v("NALU:SLICE_A slice_idr", currStream);
        break;
      case NALU_TYPE_DPB:
        /* LC: inserting the code related to DP processing */

        currStream             = currSlice->partArr[1].bitstream;
        currStream->ei_flag    = 0;
        currStream->frame_bitoffset = 0;
        memcpy (currStream->streamBuffer, &nalu->buf[1], nalu->len-1);
        currStream->bitstream_length = RBSPtoSODB(currStream->streamBuffer, nalu->len-1);

        slice_id_b  = ue_v("NALU:SLICE_B slice_idr", currStream);
        if (active_pps->redundant_pic_cnt_present_flag)
          redundant_pic_cnt_b = ue_v("NALU:SLICE_B redudand_pic_cnt", currStream);
        else
          redundant_pic_cnt_b = 0;
        
        /* LC: resilience code to be inserted */
        /*         FreeNALU(nalu); */
        /*         return current_header; */

        break;
      case NALU_TYPE_DPC:
        /* LC: inserting the code related to DP processing */
        currStream             = currSlice->partArr[2].bitstream;
        currStream->ei_flag    = 0;
        currStream->frame_bitoffset = 0;
        memcpy (currStream->streamBuffer, &nalu->buf[1], nalu->len-1);
        currStream->bitstream_length = RBSPtoSODB(currStream->streamBuffer, nalu->len-1);
        
        slice_id_c  = ue_v("NALU:SLICE_C slice_idr", currStream);
        if (active_pps->redundant_pic_cnt_present_flag)
          redundant_pic_cnt_c = ue_v("NALU:SLICE_C redudand_pic_cnt", currStream);
        else
          redundant_pic_cnt_c = 0;
        /* LC: resilience code to be inserted */

        FreeNALU(nalu);
        return current_header;

        break;
      case NALU_TYPE_SEI:
//        printf ("read_new_slice: Found NALU_TYPE_SEI, len %d\n", nalu->len);
        InterpretSEIMessage(nalu->buf,nalu->len,img);
        break;
      case NALU_TYPE_PPS:
        ProcessPPS(nalu);
        break;

      case NALU_TYPE_SPS:
        ProcessSPS(nalu);
        break;
      case NALU_TYPE_PD:
//        printf ("read_new_slice: Found 'Access Unit Delimiter' NAL unit, len %d, ignored\n", nalu->len);
        break;
      case NALU_TYPE_EOSEQ:
//        printf ("read_new_slice: Found 'End of Sequence' NAL unit, len %d, ignored\n", nalu->len);
        break;
      case NALU_TYPE_EOSTREAM:
//        printf ("read_new_slice: Found 'End of Stream' NAL unit, len %d, ignored\n", nalu->len);
        break;
      case NALU_TYPE_FILL:
//        printf ("read_new_slice: Found NALU_TYPE_FILL, len %d\n", nalu->len);
//        printf ("Skipping these filling bits, proceeding w/ next NALU\n");
        break;
      default:
        printf ("Found NALU type %d, len %d undefined, ignore NALU, moving on\n", nalu->nal_unit_type, nalu->len);
    }
  }

  FreeNALU(nalu);

  return  current_header;
}
/*!
 ************************************************************************
 * \brief
 *    Reads new slice from bit_stream
 ************************************************************************
 */
int read_new_slice()
{
  NALU_t *nalu = AllocNALU(MAX_CODED_FRAME_SIZE);
  int current_header;
  int ret;
  int BitsUsedByHeader;
  Slice *currSlice = img->currentSlice;
  Bitstream *currStream;
  int newframe;

  int slice_id_a, slice_id_b, slice_id_c;
  int redundant_pic_cnt_b, redundant_pic_cnt_c;
  long ftell_position;

  while (1)
  {
    ftell_position = ftell(bits);

    if (FileFormat == PAR_OF_ANNEXB)
      ret=GetAnnexbNALU (nalu);
    else
      ret=GetRTPNALU (nalu);

	//NALUtoRBSP(nalu);
   assert (nalu != NULL);
   //EBSPtoRBSP (nalu->buf, nalu->len) ; 
    
    if (ret < 0)
		exit (-1);
    else if (ret == 0)
    {
        return EOS;
    }

    switch (nalu->nal_unit_type)
    {
      case NALU_TYPE_SLICE:
      case NALU_TYPE_IDR:
        img->idr_flag = (nalu->nal_unit_type == NALU_TYPE_IDR);
        img->nal_reference_idc = nalu->nal_reference_idc;
        img->disposable_flag = (nalu->nal_reference_idc == NALU_PRIORITY_DISPOSABLE);
        currSlice->dp_mode = PAR_DP_1;
        currSlice->max_part_nr = 1;
        currSlice->ei_flag = 0;
        currStream = currSlice->partArr[0].bitstream;
        currStream->ei_flag = 0;
        currStream->frame_bitoffset = 0;
        memcpy (currStream->streamBuffer, &nalu->buf[1], nalu->len-1);
        currStream->bitstream_length = RBSPtoSODB(currStream->streamBuffer, nalu->len-1);

        // Some syntax of the Slice Header depends on the parameter set, which depends on
        // the parameter set ID of the SLice header.  Hence, read the pic_parameter_set_id
        // of the slice header first, then setup the active parameter sets, and then read
        // the rest of the slice header
        BitsUsedByHeader = FirstPartOfSliceHeader();
        UseParameterSet (currSlice->pic_parameter_set_id);
        BitsUsedByHeader+= RestOfSliceHeader ();

        FmoInit (active_pps, active_sps);

        init_lists(img->type);
        reorder_lists (img->type, img->currentSlice);

        // From here on, active_sps, active_pps and the slice header are valid
        img->current_mb_nr = currSlice->start_mb_nr;

        if (img->tr_old != img->ThisPOC)
        {
          newframe=1;
          img->tr_old = img->ThisPOC;
		  current_header = SOP;
        }
        else
		{
          newframe = 0;
		  current_header = SOS;
		}
		g_new_frame=0;

       FreeNALU(nalu);
        return current_header;
        break;
      case NALU_TYPE_DPA:
        //! The state machine here should follow the same ideas as the old readSliceRTP()
        //! basically:
        //! work on DPA (as above)
        //! read and process all following SEI/SPS/PPS/PD/Filler NALUs
        //! if next video NALU is dpB, 
        //!   then read and check whether it belongs to DPA, if yes, use it
        //! else
        //!   ;   // nothing
        //! read and process all following SEI/SPS/PPS/PD/Filler NALUs
        //! if next video NALU is dpC
        //!   then read and check whether it belongs to DPA (and DPB, if present), if yes, use it, done
        //! else
        //!   use the DPA (and the DPB if present)

        /* 
            LC: inserting the code related to DP processing, mainly copying some of the parts
            related to NALU_TYPE_SLICE, NALU_TYPE_IDR.
        */


        img->idr_flag          = (nalu->nal_unit_type == NALU_TYPE_IDR);
        img->nal_reference_idc = nalu->nal_reference_idc;
        img->disposable_flag   = (nalu->nal_reference_idc == NALU_PRIORITY_DISPOSABLE);
        currSlice->dp_mode     = PAR_DP_3;
        currSlice->max_part_nr = 3;
        currSlice->ei_flag     = 0;
        currStream             = currSlice->partArr[0].bitstream;
        currStream->ei_flag    = 0;
        currStream->frame_bitoffset = 0;
        memcpy (currStream->streamBuffer, &nalu->buf[1], nalu->len-1);
        currStream->bitstream_length = RBSPtoSODB(currStream->streamBuffer, nalu->len-1);
        
        BitsUsedByHeader     = FirstPartOfSliceHeader();
        UseParameterSet (currSlice->pic_parameter_set_id);
        BitsUsedByHeader    += RestOfSliceHeader ();
        
        FmoInit (active_pps, active_sps);
        
        init_lists(img->type);
        reorder_lists (img->type, img->currentSlice);

        // From here on, active_sps, active_pps and the slice header are valid
        img->current_mb_nr = currSlice->start_mb_nr;

        if (img->tr_old != img->ThisPOC)
        {
          newframe=1;
          img->tr_old = img->ThisPOC;
		  current_header = SOP;
        }
        else
		{
          newframe = 0;
		  current_header = SOS;
		}
		g_new_frame=0;

        /* 
           LC:
              Now I need to read the slice ID, which depends on the value of 
              redundant_pic_cnt_present_flag (pag.49). 
        */

        slice_id_a  = ue_v("NALU:SLICE_A slice_idr", currStream);
        break;
      case NALU_TYPE_DPB:
        /* LC: inserting the code related to DP processing */

        currStream             = currSlice->partArr[1].bitstream;
        currStream->ei_flag    = 0;
        currStream->frame_bitoffset = 0;
        memcpy (currStream->streamBuffer, &nalu->buf[1], nalu->len-1);
        currStream->bitstream_length = RBSPtoSODB(currStream->streamBuffer, nalu->len-1);

        slice_id_b  = ue_v("NALU:SLICE_B slice_idr", currStream);
        if (active_pps->redundant_pic_cnt_present_flag)
          redundant_pic_cnt_b = ue_v("NALU:SLICE_B redudand_pic_cnt", currStream);
        else
          redundant_pic_cnt_b = 0;
        
        /* LC: resilience code to be inserted */
        /*         FreeNALU(nalu); */
        /*         return current_header; */

        break;
      case NALU_TYPE_DPC:
        /* LC: inserting the code related to DP processing */
        currStream             = currSlice->partArr[2].bitstream;
        currStream->ei_flag    = 0;
        currStream->frame_bitoffset = 0;
        memcpy (currStream->streamBuffer, &nalu->buf[1], nalu->len-1);
        currStream->bitstream_length = RBSPtoSODB(currStream->streamBuffer, nalu->len-1);
        
        slice_id_c  = ue_v("NALU:SLICE_C slice_idr", currStream);
        if (active_pps->redundant_pic_cnt_present_flag)
          redundant_pic_cnt_c = ue_v("NALU:SLICE_C redudand_pic_cnt", currStream);
        else
          redundant_pic_cnt_c = 0;
        /* LC: resilience code to be inserted */

        FreeNALU(nalu);
        return current_header;

        break;
      case NALU_TYPE_SEI:
//        printf ("read_new_slice: Found NALU_TYPE_SEI, len %d\n", nalu->len);
        InterpretSEIMessage(nalu->buf,nalu->len,img);
        break;
      case NALU_TYPE_PPS:
        ProcessPPS(nalu);
        break;

      case NALU_TYPE_SPS:
        ProcessSPS(nalu);
        break;
      case NALU_TYPE_PD:
//        printf ("read_new_slice: Found 'Access Unit Delimiter' NAL unit, len %d, ignored\n", nalu->len);
        break;
      case NALU_TYPE_EOSEQ:
//        printf ("read_new_slice: Found 'End of Sequence' NAL unit, len %d, ignored\n", nalu->len);
        break;
      case NALU_TYPE_EOSTREAM:
//        printf ("read_new_slice: Found 'End of Stream' NAL unit, len %d, ignored\n", nalu->len);
        break;
      case NALU_TYPE_FILL:
//        printf ("read_new_slice: Found NALU_TYPE_FILL, len %d\n", nalu->len);
//        printf ("Skipping these filling bits, proceeding w/ next NALU\n");
        break;
      default:
        printf ("Found NALU type %d, len %d undefined, ignore NALU, moving on\n", nalu->nal_unit_type, nalu->len);
    }
  }

  FreeNALU(nalu);

  return  current_header;
}


/*!
 ************************************************************************
 * \brief
 *    Initializes the parameters for a new frame
 ************************************************************************
 */
void init_frame(struct img_par *img)
{
  int i,k;

  if (dec_picture)
  {
    // this may only happen on slice loss
    store_picture_in_dpb(dec_picture);
  }

  dec_picture = alloc_storable_picture (img->width, img->height, img->width_cr, img->height_cr);
  dec_picture->poc=img->framepoc;
  dec_picture->top_poc=img->toppoc;
  dec_picture->bottom_poc=img->bottompoc;
  dec_picture->pic_num = img->frame_num;

  for (i=0;i<listXsize[LIST_0];i++)
  {
    dec_picture->ref_pic_num[LIST_0][i]=listX[LIST_0][i]->poc * 2 ; 
  }
    
  img->current_slice_nr=0;

  if (img->type > SI_SLICE)
  {
    set_ec_flag(SE_PTYPE);
    img->type = P_SLICE;  // concealed element
  }

  // allocate memory for frame buffers
  if (img->number == 0) 
  {
    init_global_buffers(img); 
  }

  // CAVLC init
  for (i=0;i < (int)img->PicSizeInMbs; i++){
    for (k=0;k<4;k++)
      memset( img->nz_coeff[i][k],-1,6*sizeof(int));  // CAVLC
  }

  if(img->constrained_intra_pred_flag)
  {
	  memset(img->intra_block,1,(int)img->PicSizeInMbs*sizeof(int));
  }

  // WYK: Oct. 8, 2001. Set the slice_nr member of each MB to -1, to ensure correct when packet loss occurs
  // TO set Macroblock Map (mark all MBs as 'have to be concealed')
  for(i=0; i<(int)img->PicSizeInMbs; i++)
  {
    img->mb_data[i].slice_nr = -1; 
    img->mb_data[i].ei_flag = 1;
  }
}

void reset_wp_params(struct img_par *img)
{
  int i;
  int log_weight_denom;

  for (i=0; i<MAX_REFERENCE_PICTURES; i++)
  {
/*    for (comp=0; comp<3; comp++)
    {
      log_weight_denom = (comp == 0) ? img->luma_log2_weight_denom : img->chroma_log2_weight_denom;
      img->wp_weight[0][i][comp] = 1<<log_weight_denom;
      img->wp_weight[1][i][comp] = 1<<log_weight_denom;
    }*/
	  log_weight_denom =img->luma_log2_weight_denom;
      img->wp_weight[0][i][0] = 1<<log_weight_denom;
      img->wp_weight[1][i][0] = 1<<log_weight_denom;

	  log_weight_denom = img->chroma_log2_weight_denom;
      img->wp_weight[0][i][1] = img->wp_weight[0][i][2] = 1<<log_weight_denom;
      img->wp_weight[1][i][1] = img->wp_weight[1][i][2] = 1<<log_weight_denom;
  }
}
